@@ -1,5 +1,7 @@
 import argparse
 import csv
+import os.path
+
 import requests
 import subprocess
 import sys
@@ -8,17 +10,54 @@ from typing import List, Optional
 from urllib.parse import urlparse
 
 
-def download_file(file_url, md5: bytes):
+DOWNLOAD_ATTEMPTS = 3
+
+
+def get_file_md5(path: str) -> bytes:
+    return subprocess.check_output(["md5sum", path]).split(b" ")[0]
+
+
+def download_file(file_url, md5: bytes, already_downloaded: dict):
     file_name = file_url.split("/")[-1]
+
+    if file_name not in already_downloaded:
+        already_downloaded[file_name] = 1
+    else:
+        already_downloaded[file_name] += 1
+        file_name_parts = file_name.split(".")
+        file_name_parts[0] += f"_{already_downloaded[file_name]}"
+        file_name = ".".join(file_name_parts)
+
+    if os.path.exists(file_name):
+        h = get_file_md5(file_name)
+        if h == md5:
+            sys.stdout.write(f"Skipping {file_name} (already exists with correct checksum)\n")
+        else:
+            sys.stdout.write(f"Error: encountered conflicting existing file with name {file_name}\n")
+        return
+
     sys.stdout.write(f"Downloading and verifying {file_name}... ")
     sys.stdout.flush()
-    subprocess.check_output(["wget", file_url], stderr=subprocess.PIPE)
-    h = subprocess.check_output(["md5sum", file_name]).split(b" ")[0]
-    if h != md5:
-        sys.stdout.write(f"hash mismatch: downloaded file hash '{h}' != recorded hash '{md5}'")
-        exit(1)
-    assert h == md5
-    sys.stdout.write("done.")
+
+    attempts = 1
+
+    while True:
+        subprocess.check_output(["wget", "-O", file_name, file_url], stderr=subprocess.PIPE)
+        h = get_file_md5(file_name)
+
+        if h == md5:
+            break
+
+        sys.stdout.write(f"\n\thash mismatch: downloaded file hash '{h}' != recorded hash '{md5}'\n")
+        attempts += 1
+        if attempts > DOWNLOAD_ATTEMPTS:
+            sys.stdout.write("VERIFICATION FAILED (MULTIPLE HASH MISMATCH.)\n")
+            return
+
+        sys.stdout.write(f"\tretrying (attempt {attempts} of {DOWNLOAD_ATTEMPTS})\n")
+        subprocess.check_output(["rm", "-f", file_name])
+
+    sys.stdout.write("done.\n")
     sys.stdout.flush()
 
 
@@ -52,14 +91,14 @@ def main(args: Optional[List[str]] = None):
         with open(index, "r") as fh:
             index_contents = fh.read()
 
+    already_downloaded = {}
     index_reader = csv.DictReader(index_contents, delimiter="\t")
 
     for row in index_reader:
         if "FASTQ" in row:
-            download_file(row["FASTQ"], bytes(row["FASTQ_MD5"], encoding="ascii"))
-
+            download_file(row["FASTQ"], bytes(row["FASTQ_MD5"], encoding="ascii"), already_downloaded)
         if "PAIRED_FASTQ" in row:
-            download_file(row["PAIRED_FASTQ"], bytes(row["PAIRED_FASTQ_MD5"], encoding="ascii"))
+            download_file(row["PAIRED_FASTQ"], bytes(row["PAIRED_FASTQ_MD5"], encoding="ascii"), already_downloaded)
 
 
 if __name__ == "__main__":
